<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="description" content="">
    <meta name="author" content="">


    <title>mitmproxy</title>
    <link href="/static/bootstrap.min.css" rel="stylesheet">
    <link href="/static/mitmproxy.css" rel="stylesheet">
    <link href="/static/fontawesome/css/font-awesome.min.css" rel="stylesheet">
    <link rel="icon" href="/static/images/favicon.ico" type="image/x-icon"/>
  </head>

  <body>
<div class="navbar navbar-default" role="navigation">
    <div class="container">
        <div class="navbar-header">
          <a class="navbar-brand" href="#">
                       <img height="20px" src="static/images/mitmproxy-long.png"/>
          </a>
        </div>
    </div>
</div>

<div class="container">
    {% block content %}
    {% endblock %}
</div>

  </body>
</html>
